.combo-box-popup > .list-view , .context-menu {
 	-fx-background-color: -pdfsam-light-border-color, -pdfsam-tooltipbg-color;
    -fx-background-insets: 0.0, 0.5;
    -fx-background-radius: 0.0;
	-fx-effect: none;
}
.pdfsam-input-password {
	-fx-background-color: -pdfsam-box-border-color, -pdfsam-pane-bg;
    -fx-background-insets: 0.0, 0.4;
    -fx-background-radius: 0.0;
	-fx-effect: none;
    -fx-border-style: none;
}

.pdfsam-help-popup {
    -fx-background-radius: 0.0;
	-fx-effect: none;
}