/**
 * Banner
 */
.pdfsam-banner{
 	-fx-padding: 0.0;
 	-fx-background-color: -pdfsam-headerbg-color;
    -fx-background-insets: 0.0;
    -fx-background-radius: 3.0;
    -fx-alignment: bottom-left;  
    -fx-effect: -pdfsam-shadow-effect;
}

.pdfsam-logo{
	-fx-padding: 8.0 4.0 8.0 12.0;
 	-fx-spacing: 15.0px;
 	-fx-alignment: center-left;    
}
.pdfsam-banner-buttons{
	-fx-padding: 0.0 3.0em 0.0 0.0;
 	-fx-spacing:  0.416667em;
 	-fx-alignment: center-right;
 }
.pdfsam-toolbar-button, .pdfsam-toolbar-button:focused {
    -fx-background-color: -pdfsam-headerbg-color;
    -fx-background-insets: 0.0;
    -fx-background-radius: 3.0;
}
.pdfsam-toolbar-button:focused {
    -fx-background-color: -pdfsam-focus-border-color, -pdfsam-headerbg-color;
    -fx-background-insets: 0.0, 0.5;
    -fx-background-radius: 3.0;	
}
.pdfsam-toolbar-button:hover{
    -fx-background-color: -pdfsam-box-border-color, -pdfsam-flat-bg-color;
    -fx-background-insets: 0.0, 0.3;
    -fx-background-radius: 3.0;  
    -fx-translate-y: -1.0;
	-fx-effect: -pdfsam-shadow-effect;
}
.pdfsam-toolbar-button:armed{
	-fx-background-color: -pdfsam-armed-button-color;
    -fx-background-insets: 0.0;
    -fx-translate-y: -2.0;
	-fx-effect: -pdfsam-armed-shadow-effect;
	-fx-opacity: 0.7;
}
.pdfsam-toolbar-button:disabled {
    -fx-opacity: -fx-disabled-opacity;
}
.logs-notification {
	-fx-background-color: -pdfsam-error-color;
	-fx-background-insets:  0.0;
    -fx-background-radius: 2.0px;
	-fx-font-size: 11.0px;
	-fx-font-weight: bold;
	-fx-padding: 1.5 2.5 1.5 2.5;
	-fx-translate-y: -6.0px;
	-fx-translate-x: 3.0px;
	-fx-text-fill: #FFFFFF;
	-fx-opacity: 0.95;
	-fx-effect: -pdfsam-shadow-effect;
}
.header-title{
	-fx-font-weight: bold; 
	-fx-font-style: italic;
	-fx-text-fill: -pdfsam-light-text-color;
	-fx-padding: 0.666667em 0.333333em 0.666667em 0.083333em;
}