#pdfsam-main-pane{
	-fx-padding: 0.333333em 0.5em 0.0 0.5em;
}
#work-area, #pdfsam-dashboard {
	-fx-padding: 0.666667em 0.166667em 0.666667em 0.166667em;
}
/**
 * Log
 */
#log-view, #log-view:focused{
	-fx-background-color: -pdfsam-box-border-color, -pdfsam-tooltipbg-color;
    -fx-background-insets: 0.0, 0.4;
    -fx-background-radius: 3.0;
	-fx-effect: -pdfsam-shadow-effect !important;
    -fx-padding: 0.083333em;
}
#log-view > .virtual-flow > .scroll-bar:vertical{
   -fx-padding: 0.083333em;
}
#log-view .list-cell,.list-cell:odd {
}

.info-log{
	-fx-text-fill: -pdfsam-dark-text-color;
}

.warn-log{
	-fx-text-fill: derive(-pdfsam-info-color, -30.0%);
}

.error-log{
	-fx-text-fill: -pdfsam-error-color;
}

.log-has-errors > .glyph-icon  {
    -fx-fill: -pdfsam-error-color;
}
/**
 * Info pane
 */
.info-props{
	-fx-background-color: -pdfsam-box-border-color, -pdfsam-pane-bg;
    -fx-background-insets: 0.0, 0.4;
	-fx-background-radius: 0.25em, 0.25em;
	-fx-padding: 1.25em;
	-fx-hgap: 0.833333em;
	-fx-vgap: 0.833333em;
}
.info-property-value, .info-property {
	-fx-background-color: #FFFFFF;
    -fx-background-insets:  0.0;	
	-fx-border-radius: 0.0;
}
.info-property {
	-fx-font-weight: bold;
}
/**
 * Tab pane
 */
.tab-pane > .tab-header-area > .headers-region > .tab:left {
	-fx-background-color: -pdfsam-defaultbg-color, -pdfsam-box-border-color, -pdfsam-pane-bg;
    -fx-background-insets: 0.0, 1.0, 1.4;
    -fx-background-radius: 3.0, 3.0, 3.0;
    -fx-opacity: 0.8;
}
.tab-pane > .tab-header-area > .headers-region > .tab:selected {
    -fx-background-color: -pdfsam-defaultbg-color, -pdfsam-theme-color, -pdfsam-pane-bg;
    -fx-background-insets: 0.0, 1.0, 1.4;
    -fx-background-radius: 3.0, 3.0, 3.0;
    -fx-effect: -pdfsam-shadow-effect;
    -fx-opacity: 1.0;
}
.tab-pane:focused > .tab-header-area > .headers-region > .tab:selected .focus-indicator {
    -fx-border-width: 0.0;
}
.tab-pane > .tab-header-area > .tab-header-background {
    -fx-background-color: -pdfsam-defaultbg-color;
    -fx-background-insets: 0.0;
    -fx-background-radius: 0.0;
    -fx-effect: null;
}